/*Prima di proseguire con la scrittura del codice
 *devi settare i pin a cui andrai a collegare il
 *display LCD.
 * N.B. la libreria in questione controlla il display
 * LCD con solo 4bits.
 */

#define RS PORTAbits.RA0
#define EN PORTAbits.RA1
#define D4 PORTAbits.RA2
#define D5 PORTAbits.RA3
#define D6 PORTAbits.RA4
#define D7 PORTAbits.RA5

#include <stdio.h>

void LCD_PORT(unsigned char a) {
    if ((a & 1) == 1)
        D4 = 1;
    else
        D4 = 0;
    if ((a & 2) == 2)
        D5 = 1;
    else
        D5 = 0;
    if ((a & 4) == 4)
        D6 = 1;
    else
        D6 = 0;
    if ((a & 8) == 8)
        D7 = 1;
    else
        D7 = 0;
}

void LCD_ScriviComando(unsigned char Comando) {
    RS = 0;
    LCD_PORT(Comando);
    EN = 1;
    __delay_ms(4);
    EN = 0;
}

void LCD_Display(unsigned char Stato, unsigned char Cursore, unsigned char LampeggioCursore) {
    LCD_ScriviComando(0x00);
    unsigned char Comando = 0x08 | Stato << 2 | Cursore << 1 | LampeggioCursore;
    LCD_ScriviComando(Comando);
}

void InizializzaLCD(void) {
    // Istruzioni per l'inizializzazione del display (vedi datasheet)
    LCD_PORT(0x00);
    __delay_ms(20);
    LCD_ScriviComando(0x03);
    __delay_ms(10);
    LCD_ScriviComando(0x03);
    __delay_ms(10);
    LCD_ScriviComando(0x03);
    // Funzionamento a 4 bit
    LCD_ScriviComando(0x02);
    // Display OFF
    LCD_ScriviComando(0x02);
    LCD_ScriviComando(0x08);
    // Display CLEAR
    LCD_ScriviComando(0x00);
    LCD_ScriviComando(0x0C);
    // Modalit: incremento con nessuno spostamento
    LCD_ScriviComando(0x00);
    LCD_ScriviComando(0x06);
}

void LCD_Cancella(void) {
    // Cancello il display
    LCD_ScriviComando(0x00);
    // Riporto il cursore alla casella iniziale
    LCD_ScriviComando(0x01);
}

void LCD_Cursore(unsigned char Riga, unsigned char Colonna) {
    unsigned char Dato;
    if (Riga == 1) {
        Dato = 0x80 | Colonna - 1;
        LCD_ScriviComando(Dato >> 4);
        LCD_ScriviComando(Dato & 0x0F);
    }
    if (Riga == 2) {
        Dato = 0xC0 | Colonna - 1;
        LCD_ScriviComando(Dato >> 4);
        LCD_ScriviComando(Dato & 0x0F);
    }
}

void LCD_ScriviCarattere(unsigned char Carattere) {
    RS = 1;
    LCD_PORT(Carattere >> 4);
    EN = 1;
    __delay_us(40);
    EN = 0;
    LCD_PORT(Carattere & 0x0F);
    EN = 1;
    __delay_us(40);
    EN = 0;
}

void LCD_ScriviStringa(char *Stringa) {
    for (unsigned char i = 0; Stringa[i] != '\0'; i++)
        LCD_ScriviCarattere(Stringa[i]);
}

void LCD_ScriviIntero(int Numero) {
    char Buffer[16];
    sprintf(Buffer, "%i", Numero);
    LCD_ScriviStringa(Buffer);
}

void LCD_ShiftaDestra(void) {
    LCD_ScriviComando(0x01);
    LCD_ScriviComando(0x0C);
}

void LCD_ShiftaSinistra(void) {
    LCD_ScriviComando(0x01);
    LCD_ScriviComando(0x08);
}

